/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package com.l2jserver.gameserver.communitybbs.Manager;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import com.l2jserver.Config;
import com.l2jserver.L2DatabaseFactory;
import com.l2jserver.gameserver.Announcements;
import com.l2jserver.gameserver.GameServer;
import com.l2jserver.gameserver.Shutdown;
import com.l2jserver.gameserver.cache.HtmCache;
import com.l2jserver.gameserver.datatables.MapRegionTable;
import com.l2jserver.gameserver.instancemanager.CastleManager;
import com.l2jserver.gameserver.instancemanager.TownManager;
import com.l2jserver.gameserver.instancemanager.ZoneManager;
import com.l2jserver.gameserver.model.BlockList;
import com.l2jserver.gameserver.model.L2Object;
import com.l2jserver.gameserver.model.L2Party;
import com.l2jserver.gameserver.model.L2World;
import com.l2jserver.gameserver.model.Location;
import com.l2jserver.gameserver.model.actor.instance.L2PcInstance;
import com.l2jserver.gameserver.model.entity.Castle;
import com.l2jserver.gameserver.model.zone.L2ZoneType;
import com.l2jserver.gameserver.model.zone.type.L2JailZone;
import com.l2jserver.gameserver.network.L2GameClient;
import com.l2jserver.gameserver.network.SystemMessageId;
import com.l2jserver.gameserver.network.serverpackets.AskJoinParty;
import com.l2jserver.gameserver.network.serverpackets.NpcHtmlMessage;
import com.l2jserver.gameserver.network.serverpackets.ShowBoard;
import com.l2jserver.gameserver.network.serverpackets.SystemMessage;
import com.l2jserver.gameserver.util.TextReplacer;

/**
 * @author  TEN / nanasisaso / TSL / JOJO
 */
public class FavoriteManager extends BaseBBSManager
{
	private static Logger _log = Logger.getLogger(FavoriteManager.class.getName());
	private static FavoriteManager _instance = new FavoriteManager();

	private static final int CUSTOM_SERVER_RESTART_MINIMUM = 3;
	private static boolean _autorestart = false;
	private CustomServerRestart restart;
	class CustomServerRestart {
		int total;		//T[o[S
		int score;		//X^]҂̐l
		int rate;		//
	}

	/**
	 * @return
	 */
	public static FavoriteManager getInstance()
	{
		return _instance;
	}

	/* (non-Javadoc)
	 * @see com.l2jserver.gameserver.communitybbs.Manager.BaseBBSManager#parsecmd(java.lang.String, com.l2jserver.gameserver.model.actor.instance.L2PcInstance)
	 */
	@Override
	public void parsecmd(String command, L2PcInstance activeChar)
	{
		if (activeChar == null) return;

		if (command.equals("_bbsgetfav"))
		{
			showFavorite(activeChar);
		}
		else if (command.startsWith("_bbsgetfav;"))
		{
			StringTokenizer st = new StringTokenizer(command, ";");
			st.nextToken();
			String subcmd = st.nextToken();
			//T[oċN\
			if (subcmd.equals("restart_srv"))
			{
				doRestartServer(activeChar);
				parsecmd("_bbsgetfav", activeChar);
			}
			//LN^[\
			else if (subcmd.equals("repair_char"))
			{
				int obj_id = Integer.parseInt(st.nextToken());
				doRepairChar(activeChar, obj_id);
				parsecmd("_bbsgetfav", activeChar);
			}
			//ȈՃp[eB[}b`
			else if (subcmd.equals("bbs_party_match"))
			{
				StringBuilder htmlCode = new StringBuilder("<html><body><br>");
				subcmd = st.nextToken();
				if (subcmd.startsWith("_menu"))
				{
					int roomId = getRoomIdByCharId(activeChar.getObjectId());
					if (roomId != -1)
					{
						//Qς݂ȂQĂ\B
						htmlCode.append(showPartyRoom(activeChar, roomId));
					}
					else
					{
						String[] values = subcmd.split(" ");
						//QȂ烊Xg\B
						if (values.length == 3)
							htmlCode.append(showPartyRoomList(activeChar, values[1], values[2]));
						else
							htmlCode.append(showPartyRoomList(activeChar, null, null));
					}
				}
				//p[eB[[J݉
				else if (subcmd.startsWith("_createRoomWindow"))
				{
					showConfigWindow(activeChar, "party/createPartyRoom.htm");
					return;
				}
				//_createRoom min max limit lootdist title
				else if (subcmd.startsWith("_createRoom"))
				{
					String[] values = subcmd.split(" ");
					//ڂĂȂ΃G[
					if (values.length < 6)
					{
						activeChar.sendMessage("p[eB[ [̊J݂Ɏs܂B");
						return;
					}
					else
					{
						try
						{
							int roomId = getRoomIdByCharId(activeChar.getObjectId());
							int min = Integer.parseInt(values[1]);
							int max = Integer.parseInt(values[2]);
							int limit = Integer.parseInt(values[3]);
							int lootdist = Integer.parseInt(values[4]);
							StringBuilder title = new StringBuilder();
							for (int i = 5; i < values.length; i++)
							{
								title.append(values[i]).append(" ");
							}
							if (roomId != -1)
								modifyPartyRoom(activeChar, roomId, min, max, limit, lootdist, title.toString());
							else
								roomId = createPartyRoom(activeChar, min, max, limit, lootdist, title.toString());
							if (roomId > -1)
								htmlCode.append(showPartyRoom(activeChar, roomId));
						}
						catch (Exception e)
						{
							e.printStackTrace();
							activeChar.sendMessage("p[eB[ [̊J݂Ɏs܂B");
							return;
						}
					}
				}
				//p[eB[[ݒύX
				else if (subcmd.startsWith("_modifyRoomWindow"))
				{
					showConfigWindow(activeChar, "party/modifyPartyRoom.htm");
					return;
				}
				//_modifyRoom roomid min max limit lootdist title
				else if (subcmd.startsWith("_modifyRoom"))
				{
					String[] values = subcmd.split(" ");
					//ڂĂȂ΃G[
					if (values.length < 7)
					{
						activeChar.sendMessage("p[eB[ [ݒ̕ύXɎs܂B");
						return;
					}
					else
					{
						try
						{
							int roomId = Integer.parseInt(values[1]);
							int min = Integer.parseInt(values[2]);
							int max = Integer.parseInt(values[3]);
							int limit = Integer.parseInt(values[4]);
							int lootdist = Integer.parseInt(values[5]);
							StringBuilder title = new StringBuilder();
							for (int i = 6; i < values.length; i++)
							{
								title.append(values[i]).append(" ");
							}
							modifyPartyRoom(activeChar, roomId, min, max, limit, lootdist, title.toString());
							htmlCode.append(showPartyRoom(activeChar, roomId));
						}
						catch (Exception e)
						{
							e.printStackTrace();
							activeChar.sendMessage("p[eB[ [ݒ̕ύXɎs܂B");
							return;
						}
					}
				}
				//_joinRoom roomid
				else if (subcmd.startsWith("_joinRoom"))
				{
					String[] values = subcmd.split(" ");
					//ڂĂȂ΃G[
					if (values.length < 2)
					{
						activeChar.sendMessage("p[eB[ [ւ̓Ɏs܂B");
						return;
					}
					else
					{
						try
						{
							int roomId = Integer.parseInt(values[1]);
							if (joinPartyRoom(activeChar, roomId))
								htmlCode.append(showPartyRoom(activeChar, roomId));
							else
							{
								activeChar.sendMessage("p[eB[ [ւ̓Ɏs܂B");
								return;
							}
						}
						catch (Exception e)
						{
							e.printStackTrace();
							activeChar.sendMessage("p[eB[ [ւ̓Ɏs܂B");
							return;
						}
					}
				}
				//_leaveRoom roomid
				else if (subcmd.startsWith("_leaveRoom"))
				{
					String[] values = subcmd.split(" ");
					//ڂĂȂ΃G[
					if (values.length < 2)
					{
						activeChar.sendMessage("p[eB[ [̑ގɎs܂B");
						return;
					}
					else
					{
						try
						{
							int roomId = Integer.parseInt(values[1]);
							leavePartyRoom(activeChar, roomId);
							htmlCode.append(showPartyRoomList(activeChar, null, null));
						}
						catch (Exception e)
						{
							e.printStackTrace();
							activeChar.sendMessage("p[eB[ [̑ގɎs܂B");
							return;
						}
					}
				}
				//_kickRoom roomid char_obj_id
				else if (subcmd.startsWith("_kickRoom"))
				{
					String[] values = subcmd.split(" ");
					//ڂĂȂ΃G[
					if (values.length < 3)
					{
						activeChar.sendMessage("p[eB[ [̋ގɎs܂B");
						return;
					}
					else
					{
						try
						{
							int roomId = Integer.parseInt(values[1]);
							int objectId = Integer.parseInt(values[2]);
							L2PcInstance character = (L2PcInstance)L2World.getInstance().findObject(objectId);
							if (character != null)
								leavePartyRoom(character, roomId);
							return;
						}
						catch (Exception e)
						{
							e.printStackTrace();
							activeChar.sendMessage("p[eB[ [̋ގɎs܂B");
							return;
						}
					}
				}
				//_joinParty roomid char_obj_id
				else if (subcmd.startsWith("_joinParty"))
				{
					String[] values = subcmd.split(" ");
					//ڂĂȂ΃G[
					if (values.length < 3)
					{
						activeChar.sendMessage("p[eBւ̊UɎs܂B");
						return;
					}
					else
					{
						try
						{
							int roomId = Integer.parseInt(values[1]);
							int objectId = Integer.parseInt(values[2]);
							L2PcInstance target = (L2PcInstance)L2World.getInstance().findObject(objectId);
							if (!requestJoinParty(activeChar, target, roomId))
								activeChar.sendMessage("p[eBւ̊UɎs܂B");
							return;
						}
						catch (Exception e)
						{
							e.printStackTrace();
							activeChar.sendMessage("p[eBւ̊UɎs܂B");
							return;
						}
					}
				}
				htmlCode.append("</body></html>");
				separateAndSend(htmlCode.toString(),activeChar);
			}
			//hbvXg
			else if (subcmd.equals("bbs_droplist"))
			{
				showConfigWindow(activeChar, "droplist/searchWindow.htm");
				return;
			}
		}
		else
		{
			ShowBoard sb = new ShowBoard("<html><body><br><br><center> the command: " + command + " is not implemented yet</center><br><br></body></html>","101");
			activeChar.sendPacket(sb);
			activeChar.sendPacket(new ShowBoard(null,"102"));
			activeChar.sendPacket(new ShowBoard(null,"103"));
		}
	}

	/* (non-Javadoc)
	 * @see com.l2jserver.gameserver.communitybbs.Manager.BaseBBSManager#parsewrite(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, com.l2jserver.gameserver.model.actor.instance.L2PcInstance)
	 */
	@Override
	public void parsewrite(String ar1, String ar2, String ar3, String ar4, String ar5, L2PcInstance activeChar)
	{
		if (activeChar == null) return;

//		ar2 = quoteString(ar2);
//		ar3 = quoteString(ar3);
//		ar4 = quoteString(ar4);
//		ar5 = quoteString(ar5);
		if (ar1.equals("droplist") && ar2.equals("search"))
		{
			separateAndSend("<html><body><br>"
					+ showDropList(activeChar, ar3, ar4, ar5)
					+ "</body></html>"
				,activeChar);
		}
		else
		{
			ShowBoard sb = new ShowBoard("<html><body><br><br><center>the command: " + ar1
				+ " is not implemented yet</center><br><br></body></html>", "101");
			activeChar.sendPacket(sb);
			activeChar.sendPacket(new ShowBoard(null, "102"));
			activeChar.sendPacket(new ShowBoard(null, "103"));
		}
	}

	/**
	 * @param activeChar
	 */
	private void showFavorite(L2PcInstance activeChar)
	{
		StringBuilder html = new StringBuilder("<html><body><br>");
		html.append("<TABLE border=0 width=610>");
		html.append("<TR><TD align=left>");
		html.append("<a action=\"bypass _bbshome\">&$377;</a>&nbsp;>&nbsp;<A action=\"bypass _bbsgetfav\">&$379;</A>");
		html.append("</TD></TR>");
		html.append("</TABLE>");
		html.append("<CENTER>");

		html.append("<IMG src=\"L2UI.squareblank\" width=\"1\" height=\"10\">");
		html.append("<TABLE border=0>");
		html.append("<TR>");
		html.append("<TD><button value=\"p[eB }b`\" action=\"bypass _bbsgetfav;bbs_party_match;_menu\" width=105 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></TD>");
		html.append("<TD><button value=\"hbv Xg\" action=\"bypass _bbsgetfav;bbs_droplist\" width=105 height=20 back=\"L2UI_ct1.button_df\" fore=\"L2UI_ct1.button_df\"></TD>");
		html.append("</TR>");
		html.append("</TABLE>");

		html.append("<IMG src=\"L2UI.squareblank\" width=\"1\" height=\"10\">");
		html.append("<IMG src=\"L2UI.SquareWhite\" width=\"610\" height=\"1\">");
		html.append("<IMG src=\"L2UI.squareblank\" width=\"1\" height=\"10\">");

		html.append("<font color=\"LEVEL\">T[o[ċN\</font><BR>");
		html.append("<TABLE border=0 cellspacing=0 cellpadding=0 width=610>");
		html.append("<TR>");
		html.append("<TD fixWIDTH=5></td>");
		html.append("<TD fixWIDTH=600>");
		html.append(getRestartServerHTML(activeChar));
		html.append("</TD>");
		html.append("<TD fixWIDTH=5></td>");
		html.append("</TR>");
		html.append("</TABLE>");

		html.append("<IMG src=\"L2UI.squareblank\" width=\"1\" height=\"10\">");
		html.append("<IMG src=\"L2UI.SquareWhite\" width=\"610\" height=\"1\">");
		html.append("<IMG src=\"L2UI.squareblank\" width=\"1\" height=\"10\">");

		html.append("<font color=\"LEVEL\">LN^[\</font><BR>");
		html.append("<TABLE border=0 cellspacing=0 cellpadding=0 width=610>");
		html.append("<TR>");
		html.append("<TD fixWIDTH=5></td>");
		html.append("<TD fixWIDTH=600>");
		html.append(getRepairCharHTML(activeChar));
		html.append("</TD>");
		html.append("<TD fixWIDTH=5></td>");
		html.append("</TR>");
		html.append("</TABLE>");

		html.append("</CENTER>");
		html.append("</body>");
		html.append("</html>");
		separateAndSend(html.toString(), activeChar);
	}

	//T[oċN\ҐWv
	private void countRestartServer()
	{
		int score = 0;
		for (L2PcInstance pc : L2World.getInstance().getAllPlayers().values()) {
			L2GameClient c = pc.getClient();
			if (c == null || c.isDetached()) continue;	//s݃[hO
			if (pc.voteServerRestart) ++score;
		}

		restart.total = L2World.getInstance().getAllPlayersCount();
		restart.score = score;
		restart.rate = score * 100 / restart.total;
	}

	// \ʕ\
	public /*private*/ String getRestartServerHTML(L2PcInstance activeChar)
	{
		if (restart == null) {
			restart = new CustomServerRestart();
			countRestartServer();
		}

		StringBuilder replyMSG = new StringBuilder();
		replyMSG.append("FvC[ " + Config.CUSTOM_SERVER_RESTART_RATE + " T[oċN]ꍇLłB<br1>");
		replyMSG.append("݂̃OCF" + restart.total + " l<br1>");
		replyMSG.append("݂̊]ҐF" + restart.score + " l<br1>");
		replyMSG.append("݂̊]ҐF" + restart.rate + " <br>");

		//ċNȂΉL
		if (_autorestart)
		{
			replyMSG.append("<font color=\"LEVEL\">ɍċNĂ܂B</font><br1>");
			replyMSG.append("<font color=\"LEVEL\">SȏꏊŃOAEgĂB</font><br1>");
		}
		else
		{
			replyMSG.append("<button value=\"\\" action=\"bypass _bbsgetfav;restart_srv\" width=80 height=22 back=\"L2UI_CT1.BUTTON_DF_DOWN\" fore=\"L2UI_CT1.BUTTON_DF\"><br>");
			replyMSG.append("ӎ<br1>");
			replyMSG.append("T[oN " + com.l2jserver.util.Util.strMillTime(Config.CUSTOM_SERVER_RESTART_INTERVAL) + " o߂ĂȂƐ\ł܂B<br1>");
			replyMSG.append("܂AAJEg̐\͂P݂̂ŁAOAEgꍇ͖ƂȂ܂B<br1>");
		}
		return replyMSG.toString();
	}

	//ԊmFAʕ
	private void doRestartServer(L2PcInstance activeChar)
	{
		//ċNȂΉL
		if (_autorestart) {
			activeChar.sendMessage("ɃT[o[ċNɈڍsĂ܂B");
			return;
		}

		//R}hs҂\ςǂ`FbN
		if (activeChar.voteServerRestart) {
			activeChar.sendMessage("ɃT[o[ċN\ςłB");
			return;
		}

		// t
		if (activeChar.getLevel() < 10) {
			activeChar.sendMessage("xPOȏłȂΐ\ł܂B");
			return;
		}

		//T[oNAw莞Ԃo߂Ă邩`FbN
		if (System.currentTimeMillis() < GameServer.gameServer.serverLoadEnd + Config.CUSTOM_SERVER_RESTART_INTERVAL) {
			activeChar.sendMessage("T[oNAK莞Ԃo߂Ă܂B");
			return;
		}

		//U풆ǂ`FbN
		for (Castle castle : CastleManager.getInstance().getCastles()) {
			if (castle.getSiege().getIsInProgress()) {
				activeChar.sendMessage("U풆̓T[o[ċN\邱Ƃł܂B");
				return;
			}
		}

		activeChar.voteServerRestart = true;
		activeChar.sendMessage("T[o[ċN\t܂B");
		countRestartServer();

		if (restart.rate < Config.CUSTOM_SERVER_RESTART_RATE) {
			activeChar.sendMessage("]ҐOC " + Config.CUSTOM_SERVER_RESTART_RATE + " ɒB܂ł҂B");
			return;
		}
		if (restart.score < CUSTOM_SERVER_RESTART_MINIMUM) {
			activeChar.sendMessage(CUSTOM_SERVER_RESTART_MINIMUM + " ȏ̓ӂȂ΃T[o[ċNł܂B");
			return;
		}

		_autorestart = true;
		activeChar.sendMessage("T[o[ċN܂B");
		Announcements.getInstance().announceToAll("yT[o[ċN̂m点z");
		Announcements.getInstance().announceToAll("[VXeɂAT[o[ċN܂B");
		Announcements.getInstance().announceToAll("蓖T[o[͍ċNɈڍs܂B");
		Announcements.getInstance().announceToAll("SȏꏊŃOAEg悤ɂĂB");
		Shutdown.getInstance().startShutdown(null, Config.CUSTOM_SERVER_RESTART_TIME, true);
	}

	static final boolean CUSTOM_CHARA_REPAIR_NEAREST_TOWN = true;

	//L̃Xg\\[X
	private String getRepairCharHTML(L2PcInstance activeChar)
	{
		//AJEg̃L[Ȃ
		if (activeChar.getAccountChars().size() == 0)
		{
			return "\ȃLN^[݂܂B<br1>";
		}

		//ʃLȂ΁iʃL[ȂȂj
		StringBuilder replyMSG = new StringBuilder();
		replyMSG.append("LN^[IĉB<br1>");
if (CUSTOM_CHARA_REPAIR_NEAREST_TOWN) {{
		replyMSG.append("Ӂꏊ́AŊ̑ƂȂĂ܂B<br>");
}} else {{
		replyMSG.append("Ӂꏊ́u " + Config.CUSTOM_CHARA_REPAIR_NAME + " vƂȂĂ܂B<br>");
}}

		for (Map.Entry<Integer, String> chars : activeChar.getAccountChars().entrySet()) {
			int obj_Id = chars.getKey();
			String char_name = chars.getValue();
			replyMSG.append("<a action=\"bypass _bbsgetfav;repair_char;" + obj_Id + "\">");
			replyMSG.append(char_name);
			replyMSG.append("</a>&nbsp;");
		}
		return replyMSG.toString();
	}

	//LN^[@\̎s
	private void doRepairChar(L2PcInstance activeChar, int obj_id)
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement;

			int nx = Config.CUSTOM_CHARA_REPAIR_LOC[0], ny = Config.CUSTOM_CHARA_REPAIR_LOC[1], nz = Config.CUSTOM_CHARA_REPAIR_LOC[2];
if (CUSTOM_CHARA_REPAIR_NEAREST_TOWN) {{
			//Ŋ̑
			statement = con.prepareStatement("SELECT x,y,pkkills,accesslevel FROM characters WHERE charId=?");
			statement.setInt(1, obj_id);
			ResultSet rs = statement.executeQuery();
			if (rs.next()) {
				if (rs.getInt("accesslevel") < 0)
				{
					activeChar.sendMessage("łȂLN^[łB");
					return;
				}
				final int x = rs.getInt("x"), y = rs.getInt("y"), z = rs.getInt("z");
				Location loc = TownManager.getClosestTown(x, y).getSpawnLoc(); /*Ŋ̑*/
				nx = loc.getX(); ny = loc.getY(); nz = loc.getZ();
				if (rs.getInt("pkkills") > 5)
				{
					nx = 17817; ny = 170079; nz = -3530; /*ԃl˃tH[*/
				}
				for (L2ZoneType zone : ZoneManager.getInstance().getAllZones())
					if (zone instanceof L2JailZone && zone.isInsideZone(x, y, z))	/**/
					{
						nx = x; ny = y; nz = z;
						break;
					}
			}
			rs.close();
			statement.close();
}}
			//ړ
			statement = con.prepareStatement("UPDATE characters SET x=?,y=?,z=? WHERE charId=?");
			statement.setInt(1, nx);
			statement.setInt(2, ny);
			statement.setInt(3, nz);
			statement.setInt(4, obj_id);
			statement.execute();
			statement.close();

			//XL폜
			statement = con.prepareStatement("DELETE FROM character_skills_save WHERE charId=?");
			statement.setInt(1, obj_id);
			statement.execute();
			statement.close();

			//
			statement = con.prepareStatement("UPDATE items SET loc='INVENTORY', loc_data=0 WHERE owner_id=? AND loc='PAPERDOLL'");
			statement.setInt(1, obj_id);
			statement.execute();
			statement.close();

			activeChar.sendMessage("LN^[܂B");
			_log.info(activeChar.getAccountName() + " LN^[𕜋܂B");
		}
		catch (Exception e)
		{
			activeChar.sendMessage("LN^[Ɏs܂B");
			_log.info("Error could not repair char: " + e);
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
	}

	//ԍ̐
	private synchronized int createPartyRoomId()
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT MAX(id) FROM party_match_list");
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				return result.getInt(1) + 1;
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			return -1;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
		return 1;//1Ȃꍇ1Ԃ
	}

	//p[eB[[J݂B
	private int createPartyRoom(L2PcInstance activeChar, int min, int max, int limit, int lootdist, String title)
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("INSERT INTO party_match_list VALUES (?,?,?,?,?,?,?,?)");
			int nextId = createPartyRoomId();
			if (nextId == -1)
			{
				activeChar.sendMessage("p[eB[ [̊J݂Ɏs܂B");
				return -1;
			}
			statement.setInt(1, nextId);
			statement.setInt(2, activeChar.getObjectId());
			statement.setInt(3, min);
			statement.setInt(4, max);
			statement.setInt(5, limit);
			int castleIndex = CastleManager.getInstance().findNearestCastleIndex(activeChar);
			if (castleIndex < 0)
				statement.setString(6, "s");
			else
				statement.setString(6, CastleManager.getInstance().getCastles().get(castleIndex).getNameJA());
			statement.setInt(7, lootdist);
//			title = quoteString(title);
			statement.setString(8, title);
			statement.execute();
			statement.close();

			statement = con.prepareStatement("INSERT INTO party_match_room VALUES(?,?,?)");
			statement.setInt(1, nextId);
			statement.setInt(2, activeChar.getObjectId());
			statement.setInt(3, 0);
			statement.execute();
			statement.close();
			activeChar.sendMessage("p[eB[ [J݂܂B");
			return nextId;
		}
		catch (Exception e)
		{
			_log.info("could not create Party Room: " + e);
			activeChar.sendMessage("p[eB[ [̊J݂Ɏs܂B");
			return -1;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
	}

	//p[eB[[̐ݒύXi[_[̂݁j
	private void modifyPartyRoom(L2PcInstance activeChar, int roomId, int min, int max, int limit, int lootdist, String title)
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("UPDATE party_match_list SET min = ?, max = ?, `limit` = ?, loc = ?, type = ?, title = ? WHERE id = ?");
			statement.setInt(1, min);
			statement.setInt(2, max);
			statement.setInt(3, limit);
			int castleIndex = CastleManager.getInstance().findNearestCastleIndex(activeChar);
			if (castleIndex < 0)
				statement.setString(4, "s");
			else
				statement.setString(4, CastleManager.getInstance().getCastles().get(castleIndex).getNameJA());
			statement.setInt(5, lootdist);
//			title = quoteString(title);
			statement.setString(6, title);
			statement.setInt(7, roomId);
			statement.execute();
			statement.close();
			/*
			 * ̕ɁAύX̏𖞂ȂLN^[̑ގKv
			 *
			 */
		}
		catch (Exception e)
		{
			_log.info("could not modify Party Room: " + e);
			activeChar.sendMessage("p[eB[ [ݒ̕ύXɎs܂B");
			return;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
		brodeCastRoomInfo(activeChar, roomId);
		activeChar.sendMessage("p[eB[ [ݒύX܂B");
	}

	//p[eB[[ɓB
	private boolean joinPartyRoom(L2PcInstance activeChar, int roomId)
	{
		PartyRoomInfo pi = getPartyRoomInfo(roomId);
		if (pi.limit == -1 || pi.count == -1)
		{
			activeChar.sendMessage("[܂B");
			return false;
		}
		else if (pi.limit <= pi.count)
		{
			activeChar.sendMessage("l̏ɒBĂ邽߁Ał܂B");
			return false;
		}
		else if (activeChar.getLevel() < pi.min || pi.max < activeChar.getLevel())
		{
			activeChar.sendMessage("xɍĂȂ߁Ał܂B");
			return false;
		}
		else
		{
			java.sql.Connection con = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				PreparedStatement statement = con.prepareStatement("INSERT INTO party_match_room VALUES(?,?,?)");
				statement.setInt(1, roomId);
				statement.setInt(2, activeChar.getObjectId());
				if (activeChar.getParty() != null && activeChar.getParty() == ((L2PcInstance)L2World.getInstance().findObject(pi.owner_id)).getParty())
					statement.setInt(3, 1);
				else
					statement.setInt(3, 2);
				statement.execute();
				statement.close();
			}
			catch (Exception e)
			{
				_log.info("could not join Party Room: " + e);
				activeChar.sendMessage("p[eB[ [ւ̓Ɏs܂B");
				return false;
			}
			finally
			{
				try { con.close(); } catch (Exception e) {}
			}
			brodeCastRoomInfo(activeChar, roomId);
			activeChar.sendMessage("p[eB[ [֓܂B");
		}
		return true;
	}

	//p[eB[[ގ
	private void leavePartyRoom(L2PcInstance activeChar, int roomId)
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT owner_id FROM party_match_list WHERE id = ?");
			statement.setInt(1, roomId);
			ResultSet result = statement.executeQuery();
			int owner_id = 0;
			while (result.next())
			{
				owner_id = result.getInt(1);
			}
			result.close();
			statement.close();

			//ގ҂I[i[ꍇA
			if (owner_id == activeChar.getObjectId())
			{
				statement = con.prepareStatement("SELECT char_obj_id FROM party_match_room WHERE room_id = ?");
				statement.setInt(1, roomId);
				result = statement.executeQuery();
				while (result.next())
				{
					int object_id = result.getInt(1);
					L2PcInstance member = (L2PcInstance)L2World.getInstance().findObject(object_id);
					if (member != null)
					{
						separateAndSend("<html><body><br>"
								+ showPartyRoomList(member, null, null)
								+ "</body></html>"
							,member);
						member.sendMessage("p[eB[ [U܂B");
					}
				}
				result.close();
				statement.close();

				statement = con.prepareStatement("DELETE FROM party_match_list WHERE id = ?");
				statement.setInt(1, roomId);
				statement.execute();
				statement.close();
				statement = con.prepareStatement("DELETE FROM party_match_room WHERE room_id = ?");
				statement.setInt(1, roomId);
				statement.execute();
				statement.close();
			}
			else
			{
				statement = con.prepareStatement("SELECT char_obj_id FROM party_match_room WHERE room_id = ?");
				statement.setInt(1, roomId);
				result = statement.executeQuery();
				while (result.next())
				{
					int object_id = result.getInt(1);
					L2PcInstance member = (L2PcInstance)L2World.getInstance().findObject(object_id);
					if (member != null)
					{
						if (object_id == activeChar.getObjectId())
						{
							separateAndSend("<html><body><br>"
									+ showPartyRoomList(member, null, null)
									+ "</body></html>"
								,member);
							member.sendMessage("p[eB[[ގ܂B");
						}
						else
						{
							separateAndSend("<html><body><br>"
									+ showPartyRoom(member, roomId)
									+ "</body></html>"
								,member);
							member.sendMessage(activeChar.getName() + "ގ܂B");
						}
					}
				}
				result.close();
				statement.close();

				statement = con.prepareStatement("DELETE FROM party_match_room WHERE room_id = ? AND char_obj_id = ?");
				statement.setInt(1, roomId);
				statement.setInt(2, activeChar.getObjectId());
				statement.execute();
				statement.close();
			}
		}
		catch (Exception e)
		{
			_log.info("could not delete Party Room: " + e);
			activeChar.sendMessage("p[eB[ [̏IɎs܂B");
			return;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
		brodeCastRoomInfo(activeChar, roomId);
		activeChar.sendMessage("p[eB[ [I܂B");
	}

	//p[eB[ɊU
	private boolean requestJoinParty(L2PcInstance requestor, L2PcInstance target, int roomId)
	{
		PartyRoomInfo pi = getPartyRoomInfo(roomId);

		//////////////////////////////////////////////////////////////////////
		// : RequestJoinParty.java#runImpl() قڊێʂ
		if (requestor == null)
			return false;

		if (target == null)
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.TARGET_IS_INCORRECT));
			return false;
		}

		if (target.getAppearance().getInvisible())
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.TARGET_IS_INCORRECT));
			return false;
		}

		if (target.isInParty())
		{
			SystemMessage msg = new SystemMessage(SystemMessageId.C1_IS_ALREADY_IN_PARTY);
			msg.addString(target.getName());
			requestor.sendPacket(msg);
			return false;
		}

		if (BlockList.isBlocked(target, requestor))
		{
			SystemMessage sm = new SystemMessage(SystemMessageId.S1_HAS_ADDED_YOU_TO_IGNORE_LIST);
			sm.addCharName(target);
			requestor.sendPacket(sm);
			return false;
		}

		if (target == requestor)
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.INCORRECT_TARGET));
			return false;
		}

		if (target.isCursedWeaponEquipped() || requestor.isCursedWeaponEquipped())
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.INCORRECT_TARGET));
			return false;
		}

		if (target.isInJail() || requestor.isInJail())
		{
			requestor.sendMessage("Player is in Jail");
			return false;
		}

		if (target.isInOlympiadMode() || requestor.isInOlympiadMode())
			return false;
		// ܂
		//////////////////////////////////////////////////////////////////////

		//////////////////////////////////////////////////////////////////////
		// : RequestJoinParty.java#addTargetToParty() قڊێʂ
		SystemMessage msg;

		// summary of ppl already in party and ppl that get invitation
		if (!requestor.getParty().isLeader(requestor))
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.ONLY_LEADER_CAN_INVITE));
			return false;
		}
		if (requestor.getParty() != null && requestor.getParty().getMemberCount() >= 9)
//		if (requestor.getParty() != null && requestor.getParty().getMemberCount() + requestor.getParty().getPendingInvitationNumber() >= 9)
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.PARTY_FULL));
			return false;
		}
		if (requestor.getParty().getPendingInvitation())
		{
			requestor.sendPacket(new SystemMessage(SystemMessageId.WAITING_FOR_ANOTHER_REPLY));
			return false;
		}

		if (!target.isProcessingRequest())
		{
			if (requestor.getParty() == null)
				requestor.setParty(new L2Party(requestor, pi.type));
			requestor.onTransactionRequest(target);
			target.sendPacket(new AskJoinParty(requestor.getName(), pi.type));
			requestor.getParty().setPendingInvitation(true);
//			requestor.getParty().increasePendingInvitationNumber();
			
			if (Config.DEBUG)
				_log.fine("sent out a party invitation to:"+target.getName());
		}
		else
		{
			msg = new SystemMessage(SystemMessageId.C1_IS_BUSY_TRY_LATER);
			msg.addString(target.getName());
			requestor.sendPacket(msg);
			
			if (Config.DEBUG)
				_log.warning(requestor.getName() + " already received a party invitation");
		}
		// ܂
		//////////////////////////////////////////////////////////////////////

		return true;
	}

	//p[eB[[Xg\B
	private String showPartyRoomList(L2PcInstance activeChar, String optionArea, String optionLevel)
	{
		StringBuilder where_str = new StringBuilder();
		if (optionArea != null && !optionArea.equals("S"))
		{
			where_str.append(" loc = '" + optionArea + "'");
		}
		if (optionLevel != null && !optionLevel.equals("S"))
		{
			String[]v = optionLevel.split("-");
			int minlevel = Integer.parseInt(v[0]);
			int maxlevel = Integer.parseInt(v[1]);
			if (where_str.length() != 0)
				where_str.append(" AND");
			where_str.append(" min <= '" + minlevel + "' AND max >= '" + maxlevel + "'");
		}
		if (where_str.length() != 0)
			where_str.append(" WHERE" + where_str);
		where_str.append(" ORDER BY id");
		StringBuilder roomList = new StringBuilder();
		StringBuilder DelRoom = new StringBuilder();
		int roomCount = 0;
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT id FROM party_match_list" + where_str);
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				PartyRoomInfo pi = getPartyRoomInfo(result.getInt(1));
				L2PcInstance leader = (L2PcInstance)L2World.getInstance().findObject(pi.owner_id);
				if (leader == null)
				{
					if (DelRoom.length() != 0)
						DelRoom.append(",");
					DelRoom.append(pi.roomId);
				}
				else if (pi.roomId != -1 && pi.owner_id != -1)
				{
					roomList.append("<table width=610><tr>");
					roomList.append("<td width=30 align=center>" + pi.roomId + "</td>");
					roomList.append("<td width=160>" + pi.title + "</td>");
					roomList.append("<td width=120>" + leader.getName() + "</td>");
					roomList.append("<td width=120>" + pi.loc + "</td>");
					roomList.append("<td width=50 align=center>" + pi.min + " - " + pi.max + "</td>");
					roomList.append("<td width=50 align=center>" + pi.count + " / " + pi.limit + "</td>");
					roomList.append("<td width=80 align=center><button value=\"\" action=\"bypass _bbsgetfav;bbs_party_match;_joinRoom " + pi.roomId + "\" width=65 height=20 back=\"L2UI_CT1.BUTTON_DF_DOWN\" fore=\"L2UI_CT1.BUTTON_DF\"></td>");
					roomList.append("</tr></table>");
					roomCount++;
				}
			}
			result.close();
			statement.close();

			if (DelRoom.length() != 0)
			{
				statement = con.prepareStatement("DELETE FROM party_match_list WHERE id in (?)");
				statement.setString(1, DelRoom.toString());
				statement.execute();
				statement.close();
				statement = con.prepareStatement("DELETE FROM party_match_room WHERE room_id in (?)");
				statement.setString(1, DelRoom.toString());
				statement.execute();
				statement.close();
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return "p[eB[[Xg쐬ɃG[܂B";
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}

		String html = HtmCache.getInstance().getHtm(activeChar.getHtmlPrefix(), "data/html/CommunityBoard/party/partyRoomList.htm");
		html = html.replace("%roomList%", roomList);//[Xg
		return html;
	}

	//p[eB[[\B
	private /*String*/CharSequence showPartyRoom(L2PcInstance activeChar, int roomId)
	{
		PartyRoomInfo pi = getPartyRoomInfo(roomId);
		if (pi.roomId == -1 && pi.owner_id == -1)
			return "";
		StringBuilder memberList = new StringBuilder();
		int memberCount = 0;
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT * FROM party_match_room WHERE room_id = ? ORDER BY memberType");
			statement.setInt(1, roomId);
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				L2Object obj = L2World.getInstance().findObject(result.getInt(2));
				if (obj != null && obj instanceof L2PcInstance)
				{
					L2PcInstance member = (L2PcInstance)obj;
					memberList.append("<table width=610><tr>");
					memberList.append("<td width=120>" + member.getName() + "</td>");
					memberList.append("<td width=130 align=center>" + member.getTemplate().className + "</td>");
					memberList.append("<td width=40 align=center>" + member.getLevel() + "</td>");
					memberList.append("<td width=130 align=center>" + MapRegionTable.getInstance().getClosestTownName(member) + "</td>");
					if (result.getInt(3) == 0)
					{
						memberList.append("<td width=120 align=center>[[_[</td>");
					}
					else if (result.getInt(3) == 1)
					{
						memberList.append("<td width=120 align=center>p[eB[o</td>");
					}
					else
					{
						memberList.append("<td width=120 align=center>p[eB[]</td>");
					}
					if (activeChar.getObjectId() == pi.owner_id && activeChar.getObjectId() != member.getObjectId())
					{
						memberList.append("<td width=70 align=center>");
						if (result.getInt(3) == 2)
							memberList.append("<button value=\"osU\" action=\"bypass _bbsgetfav;bbs_party_match;_joinParty " + roomId + " " + member.getObjectId() + "\" width=65 height=20 back=\"L2UI_CT1.BUTTON_DF_DOWN\" fore=\"L2UI_CT1.BUTTON_DF\">");
						memberList.append("<button value=\"Ǖ\" action=\"bypass _bbsgetfav;bbs_party_match;_kickRoom " + roomId + " " + member.getObjectId() + "\" width=65 height=20 back=\"L2UI_CT1.BUTTON_DF_DOWN\" fore=\"L2UI_CT1.BUTTON_DF\">");
						memberList.append("</td>");
					}
					else
						memberList.append("<td width=70 align=center></td>");
					memberList.append("</tr></table>");
					memberCount++;
				}
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			return "p[eB[[쐬ɃG[܂B";
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}

		TextReplacer html = new TextReplacer(HtmCache.getInstance().getHtm(activeChar.getHtmlPrefix(), "data/html/CommunityBoard/party/partyRoom.htm"));
		html.replace("%roomId%", pi.roomId);	//ԍ
		html.replace("%title%", htmlescape(pi.title));	//
		html.replace("%location%", pi.loc);	//ꏊ
		html.replace("%current%", pi.count);	//݂̐l
		html.replace("%max%", pi.limit);	//̐l
		String lootdistribution = "";
		switch (pi.type)
		{
			case L2Party.ITEM_LOOTER:
				lootdistribution = "ElL";
				break;
			case L2Party.ITEM_RANDOM:
				lootdistribution = "_ɕz";
				break;
			case L2Party.ITEM_RANDOM_SPOIL:
				lootdistribution = "X|C܂߃_";
				break;
			case L2Party.ITEM_ORDER:
				lootdistribution = "ԂɎ擾";
				break;
			case L2Party.ITEM_ORDER_SPOIL:
				lootdistribution = "X|C܂ߏԂ";
				break;
		}
		html.replace("%rootType%", lootdistribution);	//^Cv
		html.replace("%minlevel%", pi.min);	//Œ჌x
		html.replace("%maxlevel%", pi.max);	//ōx
		html.replace("%memberList%", memberList);	//o[Xg
		String modifyButton = "";
		if (activeChar.getObjectId() == pi.owner_id)
			modifyButton = "<button value=\"[ݒ\" action=\"bypass _bbsgetfav;bbs_party_match;_modifyRoomWindow\" width=80 height=22 back=\"L2UI_CT1.BUTTON_DF_DOWN\" fore=\"L2UI_CT1.BUTTON_DF\">";
		html.replace("%modifyPartyRoom%", modifyButton);//ݒύX{^

		return html;
	}

	//hbvXg\B
	private /*String*/CharSequence showDropList(L2PcInstance activeChar, String optionType, String optionLevel, String optionWord)
	{
		String where_str = "";
		if (optionType.equals("ACe"))
			where_str = " AND itemname_jp.name Like ?";
		//	where_str = " AND itemname_jp.name Like '%" + quoteString(optionWord) + "%'";
		else
			where_str = " AND npc.name Like ?";
		//	where_str = " AND npc.name Like '%" + quoteString(optionWord) + "%'";
		if (!optionLevel.equals("S"))
		{
			String v[] = optionLevel.split("-");
			int minlevel = Integer.parseInt(v[0]);
			int maxlevel = Integer.parseInt(v[1]);
		//	int minlevel = Integer.parseInt(optionLevel.split("-")[0]);
		//	int maxlevel = Integer.parseInt(optionLevel.split("-")[1]);
			where_str += " AND level BETWEEN " + minlevel + " AND " + maxlevel;
		}
		_log.info("UserName: " + activeChar.getName() + " SearchWord : " + where_str);
		StringBuilder dropList = new StringBuilder();
		int dropCount = 0;
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT level, npc.name as mobname, itemname_jp.name as itemname, category"
																+ " FROM droplist,npc,itemname_jp"
																+ " WHERE droplist.mobId = npc.id"
																+ " AND droplist.itemId = itemname_jp.item_id"
																+ " AND droplist.itemId != 57"
																+ where_str
																+ " ORDER BY level, npc.name, itemname_jp.name LIMIT 20");
			statement.setString(1, "%" + optionWord + "%");
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				dropList.append("<table width=610><tr>");
				dropList.append("<td width=40 align=center>" + result.getInt("level") + "</td>");
				dropList.append("<td width=220>" + result.getString("mobname") + "</td>");
				dropList.append("<td width=300>" + result.getString("itemname") + "</td>");
				dropList.append("<td width=50 align=center>" + (result.getInt("category") == -1 ? "" : "|") + "</td>");
				dropList.append("</tr></table>");
				dropCount++;
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return "hbvXg쐬ɃG[܂B";
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}

		String html = HtmCache.getInstance().getHtm(activeChar.getHtmlPrefix(), "data/html/CommunityBoard/droplist/DropList.htm");
		html = html.replace("%dropList%", dropList);//hbvXg
		return html;
	}

	//ݒEBhE̕\
	private void showConfigWindow(L2PcInstance activeChar, String filename)
	{
		NpcHtmlMessage html = new NpcHtmlMessage(5);
		html.setFile(activeChar.getHtmlPrefix(), "data/html/CommunityBoard/" + filename);
		html.replace("%roomId%", getRoomIdByCharId(activeChar.getObjectId()));//ԍ
		activeChar.sendPacket(html);
	}

	//LN^[QĂ镔IDԂi-1j
	public int getRoomIdByCharId(int char_obj_id)
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT room_id FROM party_match_room WHERE char_obj_id = ?");
			statement.setInt(1, char_obj_id);
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				return result.getInt(1);
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			return -1;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
		return -1;
	}

	class PartyRoomInfo {
		int roomId;			//ԍ
		int owner_id = 0;	//ÎID
		int min = 1;		//Œ჌x
		int max = 100;		//ōx
		int limit = -1;		//
		String loc = "";	//p[eB̈ʒu
		int type = 0;		//ElA_
		String title = "";	//
		int count = -1;		//݂̐l
		
		PartyRoomInfo(int _roomId) { this.roomId = _roomId; }
	}

	//p[eB[[̐ݒEԂ
	public PartyRoomInfo getPartyRoomInfo(int roomId)
	{
		PartyRoomInfo pi = new PartyRoomInfo(roomId);

		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement = con.prepareStatement("SELECT owner_id, min, max, `limit`, loc, type, title FROM party_match_list WHERE id = ?");
			statement.setInt(1, roomId);
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				pi.owner_id = result.getInt(1);	//ÎID
				pi.min = result.getInt(2);		//Œ჌x
				pi.max = result.getInt(3);		//ōx
				pi.limit = result.getInt(4);	//
				pi.loc = result.getString(5);	//p[eB̈ʒu
				pi.type = result.getInt(6);		//ElA_
				pi.title = result.getString(7);	//
			}
			result.close();
			statement.close();

			statement = con.prepareStatement("SELECT COUNT(*) FROM party_match_room WHERE room_id = ?");
			statement.setInt(1, roomId);
			result = statement.executeQuery();
			while (result.next())
			{
				pi.count = result.getInt(1);	//݂̐l
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return pi;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
		return pi;
	}

	//p[eB[[̏󋵂ωAo[SɑMB
	public void brodeCastRoomInfo(L2PcInstance activeChar, int roomId)
	{
		java.sql.Connection con = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			PreparedStatement statement = con.prepareStatement("SELECT char_obj_id, memberType FROM party_match_room WHERE room_id = ?");
			statement.setInt(1, roomId);
			ResultSet result = statement.executeQuery();
			while (result.next())
			{
				int object_id = result.getInt(1);
				int member_type = result.getInt(2);
				L2PcInstance member = (L2PcInstance)L2World.getInstance().findObject(object_id);
				if (member != null)
				{
					if (member_type != 0)
					{
						PreparedStatement statement2 = con.prepareStatement("UPDATE party_match_room SET memberType = ? WHERE room_id = ? AND char_obj_id = ?");
						statement2.setInt(1, member.isInParty() ? 1 : 2);
						statement2.setInt(2, roomId);
						statement2.setInt(3, member.getObjectId());
						statement2.executeUpdate();
						statement2.close();
					}

					separateAndSend("<html><body><br>"
							+ showPartyRoom(member, roomId)
							+ "</body></html>"
						,member);
				}
			}
			result.close();
			statement.close();
		}
		catch (Exception e)
		{
			_log.info("could not update Party Room Info: " + e);
			activeChar.sendMessage("p[eB[ [̍XVɎs܂B");
			return;
		}
		finally
		{
			try { con.close(); } catch (Exception e) {}
		}
	}
}